function tile_state_init(custom_state)
    local field = custom_state:field()
    local tracked_auxprops = {}

    custom_state.on_entity_enter_func = function(self, entity)
        if not Character.from(entity) and not Obstacle.from(entity) then
            return
        end

        -- Wood characters are not harmed by brambles.
        if entity:element() == Element.Wood or entity:hittable() ~= true then return end

        if tracked_auxprops[entity:id()] then
            -- already applied
            return
        end

        -- Brambles poke for Sword damage, so they're effective against Wind
        local double_damage_prop = AuxProp.new()
            :require_hit_element(Element.Wind)
            :increase_hit_damage("DAMAGE")
        entity:add_aux_prop(double_damage_prop)

        tracked_auxprops[entity:id()] = { double_damage_prop }

        entity:hit(HitProps.new(50, Hit.Flinch | Hit.Impact, Element.Sword))
    end

    custom_state.on_entity_leave_func = function(self, entity)
        local aux_props = tracked_auxprops[entity:id()]

        if not aux_props then
            return
        end

        for _, aux_prop in ipairs(aux_props) do
            entity:remove_aux_prop(aux_prop)
        end

        tracked_auxprops[entity:id()] = nil
    end

    custom_state.change_request_func = function(self, tile)
        for id in pairs(tracked_auxprops) do
            local entity = field:get_entity(id)

            if not entity then
                tracked_auxprops[id] = nil
                goto continue
            end

            if entity:current_tile() == tile then
                local aux_props = tracked_auxprops[id]

                for _, aux_prop in ipairs(aux_props) do
                    entity:remove_aux_prop(aux_prop)
                end

                tracked_auxprops[entity:id()] = nil
            end

            ::continue::
        end

        return true
    end
end
